# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404, render, redirect
from django.utils.translation import ugettext_lazy as _
from sh.app.pages.forms import PageForm
from sh.app.pages.models import Category, Page

def view_category(request, category_slug='default'):

    category = get_object_or_404(Category,slug=category_slug)
    category_navigation = Page.objects.filter(is_active=True, category=category)

    return render(request, "pages/category.html", {
        'category': category,
        'category_navigation': category_navigation,
    })


def view_page(request, category_slug=None, page_slug=None):
    if page_slug is None:
        page = cache.get('frontpage')
        if page is None:
            page = Page.objects.get(pk=settings.FRONTPAGE_ID)
            cache.set('frontpage', page)

    else:
        key = 'page%s' % page_slug
        page = cache.get(key)
        if page is None:
            page = get_object_or_404(Page, is_active=True, slug=page_slug)
            cache.set(key, page)


    return render(request, "pages/view_page.html", {
        'page': page,
        })

@login_required
def edit_page(request, id=None):

    if id is None:
        page = Page()
    else:
        page = get_object_or_404(Page, pk=id)

    form = PageForm(instance=page)

    if request.method == 'POST':
        form = PageForm(request.POST, instance=page)
        if form.is_valid():
            page = form.save()
            messages.success(request, _('Page successfully saved'))
            return redirect(reverse('view_page', args=[page.slug]))

    return render(request, 'pages/edit_page.html', {
        'page': page,
        'form': form
    })

@login_required
def delete_page(request, id):

    if int(id) == settings.FRONTPAGE_ID:
        messages.error(request, _('This page is the front page and cannot be deleted'))
        return redirect(reverse('edit_page', args=[id]))

    page = get_object_or_404(Page, pk=id)
    if request.method == 'POST':
        if request.POST.get('delete') == 'true':

            page.delete()
            messages.success(request, _('The page was deleted'))
            return redirect(reverse('view_dashboard'))

    return render(request, 'pages/delete_page.html', {
        'page': page,
    })



# -*- coding: utf-8 -*-
import re
import urllib2
import datetime
from django.core.urlresolvers import reverse
from django.template.base import Library
from django.utils.safestring import mark_safe
from django.utils.timesince import timesince
from django.utils.translation import ugettext_lazy as _
from django.core.cache import cache
from sh.app.feed.util import IMAGE_FILE_TYPES
from sorl.thumbnail.shortcuts import get_thumbnail


register = Library()

url_re = '((http):((//)|(\\\\))+[\w\d:#@%/;$()~_?\+-=\\\.&]*)'

@register.filter
def boolean(value, arg=None):
    if value: return mark_safe('<img src="/static/icons/accept.png" style="max-width:28px;" alt="%s" />' % _('Yes'))
    elif value is None: return mark_safe('<img src="/static/icons/question.png" style="max-width:28px;" alt="%s" />' % '?')
    return mark_safe('<img src="/static/icons/delete.png" style="max-width:28px;" alt="%s"/>' % _('No'))

@register.filter
def time_since(value, arg=None):
    now = datetime.datetime.now()
    diff = now - value
    if diff.seconds < 60:
        return unicode(_('Just now'))
    return _('%s ago') % timesince(value)

def get_title_for_link(link):
    file_type = link.split(".")[len(link.split("."))-1]
    if file_type in IMAGE_FILE_TYPES:
        im = get_thumbnail(link, '100x100', crop='center', quality=99)
        return mark_safe('<img src="%s" alt="" />' % im.url)
    try:
        page = urllib2.urlopen(urllib2.Request(link)).read()
        return re.search('<title>.*</title>', page).group().replace('<title>','').replace('</title>','')
    except:
        return link



def url_text(link):
    key = re.sub('^(http|https)\://', '', link)
    key = key.replace(".","").replace("-","").replace("_","").replace("/","")
    text = cache.get(key)
    title = get_title_for_link(link)
    if not text:
        new_link = mark_safe('<a href="%s">%s</a>' % (link, title))
        cache.set(key, new_link)
        return mark_safe(new_link)
    return mark_safe(text)



@register.filter
def links(value, arg=None):
    value = unicode(value)
    url = re.search(url_re, value)
    if url:
        value = re.sub(url_re, url_text(url.group()), value)
    return mark_safe(value)

    


@register.filter
def user_mentions(value, arg=None):
    value = unicode(value)
    name = re.search('@[a-z]+', value)
    if name:
        value = re.sub('@[a-z]+', get_profile_link(name.group()), value)
    return mark_safe(value)

def get_profile_link(name):
    from django.contrib.auth.models import User
    try:
        name = name.replace("@", "")
        u = User.objects.get(username=name)
        return '<a href="%s">%s</a>' % (reverse('view_profile', args=[u.pk]), u.get_full_name())
    except (TypeError, User.DoesNotExist):
        return name



from django.conf import settings
from django.conf.urls import patterns, url, include

from django.contrib import admin
from django.http import HttpResponse
from django.utils.translation import ugettext_lazy as _

admin.autodiscover()

import os

urlpatterns = patterns('',
    url(r'^google40fdd297a3739dce.html$', lambda r:HttpResponse("google-site-verification: google40fdd297a3739dce.html")),
    url(r'^admin/django/',  include(admin.site.urls), name='django_admin'),

    url(r'dashboard/', include('sh.app.dashboard.urls')),
    url(_(r'news/'), include('sh.app.news.urls')),

    url(r'^accounts/login/$', 'django.contrib.auth.views.login', name='login'),
    url(r'^accounts/logout/$', 'sh.app.dashboard.views.logout', name='logout'),
    url(r'^fkjfbfeionafkdnfabkjf/$','sh.core.views.backup_auto'),
    url(r'^ckeditor/', include('ckeditor.urls')),

    url(r'', include('sh.app.pages.urls')),
)

if settings.DEBUG:
    urlpatterns += patterns('',
        (r'^uploads/(?P<path>.*)$', 'django.views.static.serve',
         {'document_root': settings.MEDIA_ROOT}),
        (r'^static/(?P<path>.*)$', 'django.views.static.serve',
         {'document_root': os.path.join(settings.ROOTPATH, 'files/static')}),
    )



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'NewsItem'
        db.create_table('news_newsitem', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name='newsitem_creator', to=orm['auth.User'])),
            ('saved_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name='newsitem_saved_by', to=orm['auth.User'])),
            ('date_created', self.gf('django.db.models.fields.DateTimeField')()),
            ('date_saved', self.gf('django.db.models.fields.DateTimeField')()),
            ('is_active', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=250)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=50)),
            ('pub_date', self.gf('django.db.models.fields.DateTimeField')()),
            ('sticky', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('content', self.gf('ckeditor.fields.RichTextField')()),
        ))
        db.send_create_signal('news', ['NewsItem'])


    def backwards(self, orm):
        # Deleting model 'NewsItem'
        db.delete_table('news_newsitem')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'news.newsitem': {
            'Meta': {'object_name': 'NewsItem'},
            'content': ('ckeditor.fields.RichTextField', [], {}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'newsitem_creator'", 'to': "orm['auth.User']"}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {}),
            'date_saved': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'pub_date': ('django.db.models.fields.DateTimeField', [], {}),
            'saved_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'newsitem_saved_by'", 'to': "orm['auth.User']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'sticky': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        }
    }

    complete_apps = ['news']


# -*- coding: utf-8 -*-
from django import forms
from sh.app.pages.models import Page


class PageForm(forms.ModelForm):
    class Meta:
        model = Page
        exclude = ('is_active', 'shorten_url', 'category')
        fields = ('title', 'menu_title', 'slug', 'content', 'weight')

    def __init__(self, *args, **kwargs):
        super(PageForm, self).__init__(*args, **kwargs)
        self.fields['slug'].widget.attrs['readonly'] = 'readonly'



# -*- coding: utf-8 -*-
from django.core.cache import cache
from django.template.base import Library
from sh.app.pages.models import Page

register = Library()

@register.inclusion_tag('pages/templatetags/menu.html')
def main_menu():

    pages = cache.get('mainnav')
    if pages is None:
        pages = Page.objects.all().order_by('weight')
        cache.set('mainnav', pages)

    return {'pages': pages}




# -*- coding: utf-8 -*-
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.shortcuts import render, redirect
from sh.app.news.models import NewsItem
from sh.app.pages.models import Page
from django.contrib.auth import logout as auth_logout

@login_required
def view_dashboard(request):

    pages = Page.objects.all()
    news = NewsItem.objects.all()

    return render(request, 'dashboard/base.html', {
        'pages': pages,
        'news': news,
    })

def logout(request):
    auth_logout(request)
    return redirect(reverse('view_page'))


# -*- coding: utf-8 -*-
from django import forms
from sh.app.news.models import NewsItem


class NewsItemForm(forms.ModelForm):
    class Meta:
        model = NewsItem
        fields = ('title', 'slug', 'pub_date', 'sticky','content')

    def __init__(self, *args, **kwargs):
        super(NewsItemForm, self).__init__(*args, **kwargs)
        self.fields['slug'].widget.attrs['readonly'] = 'readonly'



# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url
from django.utils.translation import ugettext_lazy as _
from sh.app.news.feeds import NewsRssFeed

urlpatterns = patterns('sh.app.news.views',
    url(r'^$', 'news_feed', name='news'),
    url(_(r'^feed/$'), NewsRssFeed()),
    url(_(r'^add/$'), 'edit_item', name='add_item'),
    url(_(r'^(?P<id>\d+)/edit/$'), 'edit_item', name='edit_item'),
    url(_(r'^(?P<item_slug>[a-z0-9-_]+)/$'), 'view_item', name='view_item'),
)



from sh.settings.base import INSTALLED_APPS

with open('/home/web/db_pw/starthjelpa', 'rb') as f:
    db_password = f.readline()
db_password = db_password.replace("\n","").strip()

DEBUG = False

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'starthjelpa',
        'USER': 'treningsportal',
        'PASSWORD': db_password,
        'HOST': '127.0.0.1',
        'PORT': ''
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
        'LOCATION': '127.0.0.1:11211',
        'TIMEOUT': 86400,
        'KEY_PREFIX': 'sh',
        }
}

INSTALLED_APPS += ( 'raven.contrib.django', )

SENTRY_DSN = 'http://d3eb55dabc6549a3b1490b25adddea62:38c31922307c4c41b9b2d384059d62b6@sentry.mocco.no/3'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'root': {
        'level': 'WARNING',
        'handlers': ['sentry'],
        },
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        },
    'handlers': {
        'sentry': {
            'level': 'ERROR',
            'class': 'raven.contrib.django.handlers.SentryHandler',
            },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        }
    },
    'loggers': {
        'django.db.backends': {
            'level': 'ERROR',
            'handlers': ['console'],
            'propagate': False,
            },
        'raven': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
            },
        'sentry.errors': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
            },
        },
    }



# -*- coding: utf-8 -*-
from datetime import datetime
from random import choice
from django.conf import settings
from django.contrib.auth.models import User
import string
from django.core.mail import send_mail
from django.db import models
from sh.core import auth

from django.contrib.contenttypes.models import ContentType

class UserProxy(User):
    class Meta:
        proxy = True

    def generate_password(self, length=8, notify=True, chars=string.letters + string.digits):
        pw = ''.join([choice(chars) for i in range(length)])
        self.set_password(pw)
        self.save()
        if notify:
            message = 'Your password is: %s' % pw
            send_mail('Your password', message, settings.EMAIL_HOST_USER, (self.email,))
            #Todo: Need notification-system first


class BaseModel (models.Model):
    created_by = models.ForeignKey(User, related_name="%(class)s_creator", editable=False)
    saved_by = models.ForeignKey(User, related_name="%(class)s_saved_by", editable=False)
    date_created = models.DateTimeField(editable=False)
    date_saved = models.DateTimeField(editable=False)
    is_active = models.BooleanField(default=True)

    def __unicode__(self):
        return self.created_by.username + ' ' + str(self.id)

    def save(self, notify_subscribers=True, *args, **kwargs):
        user = auth.get_current_user()
        if not user or not user.is_authenticated():
            user = User.objects.get(pk=1) #dummy user
        if not self.pk:
            self.created_by = user
            self.date_created = datetime.now()
        self.saved_by = user
        self.date_saved = datetime.now()
        super(BaseModel, self).save(*args,**kwargs)


    def get_absolute_url(self):
        c_type = ContentType.objects.get_for_model(self)
        return '/%s/%ss/%s/' % (str(c_type.app_label), str(c_type), str(self.pk))

    class Meta:
        abstract = True

    @classmethod
    def exclude_fields(cls):
        return ('created_by', 'saved_by', 'date_created', 'date_saved')


# -*- coding: utf-8 -*-
from datetime import datetime
from django.contrib.syndication.views import Feed
from sh.app.news.models import NewsItem

class NewsRssFeed(Feed):
    title = "Starthjelpa"
    link = "/news/"
#    description = ""

    def items(self):
        return NewsItem.objects.filter(pub_date__lte=datetime.now)[:5]

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.content


# -*- coding: utf-8 -*-
from django.contrib.auth.models import User
from django.utils import unittest

class TestCase(unittest.TestCase):

    def setUp(self):
        self.user = User.objects.create(
            id=1,
            username='dumbledore',
            first_name='Albus Percival Wulfric Brian',
            last_name='Dumbledore'
        )

    def tearDown(self):
        self.user.delete()


# -*- coding: utf-8 -*-
import os

ROOTPATH = os.path.dirname(os.path.dirname(__file__))
DEBUG_TOOLBAR = False

ADMINS = (
    ('Rolf Erik Lekang', 'me@rolflekang.com'),
    ('Ole Rønning', 'ole@myessential.net'),
)

MANAGERS = ADMINS

TIME_ZONE = 'Europe/Oslo'
LANGUAGE_CODE = 'nb'
LANGUAGES = (
    ('nb', 'Norsk'),
    ('en', 'English')
)
LOCALE_PATHS = (
    os.path.join(ROOTPATH, '../locale'),
)

SITE_URL = 'http://www.starthjelpa.no'
FRONTPAGE_ID = 1

ANALYTICS = True
ANALYTICS_TRACK_ADMINS = False
ANALYTICS_CODE = 'UA-32144222-4'

SITE_ID = 1
USE_I18N = True

MEDIA_ROOT = os.path.join(ROOTPATH, 'files/uploads')
STATIC_ROOT = os.path.join(ROOTPATH, 'static')


MEDIA_URL = '/uploads/'
STATIC_URL = '/static/'

CKEDITOR_UPLOAD_PATH = os.path.join(MEDIA_ROOT,"images")
CKEDITOR_UPLOAD_PREFIX = "/uploads/images/"
CKEDITOR_CONFIGS = {
    'default': {
        'height': 500,
        'width': 730,
        'toolbar': [
            ['Format', 'Bold', 'Italic', 'Underline','Image', 'NumberedList','BulletedList','-','Outdent','Indent', 'Link','Unlink', 'Table', 'Preview',],
            ],
        },


    }

# Additional locations of static files
STATICFILES_DIRS = (
    os.path.join(ROOTPATH, 'files/static'),
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

SECRET_KEY = 'p0otu-kbp!y74po$#kmpyd)6tx4)kq!u&)5g9jz8h)-$bnzav*'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'sh.core.auth.LocalUserMiddleware',
)

ROOT_URLCONF = 'sh.urls'

TEMPLATE_DIRS = (
    os.path.join(ROOTPATH, "templates"),
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'django.contrib.markup',

    'south',
    'sorl.thumbnail',
    'ckeditor',

    'sh.core',

    'sh.app.dashboard',
    'sh.app.pages',
    'sh.app.news',

)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from django.conf import settings

def safe_unicode(obj):
    string = str(obj)
    return string.decode('latin1').encode('utf8')

class Command (BaseCommand):

    option_list = BaseCommand.option_list

    requires_model_validation = False
    help='Run the migration'
    def handle(self, *args, **options):
        import os
        os.system('git pull origin master')
        os.system('bin/django migrate')
        path = settings.ROOTPATH.replace('/project','')
        ini_file = os.path.basename(path) + '.ini'
        os.system('touch ../uwsgi/' + ini_file)
        print ini_file + ' touched'
        print '---------------------'
        print 'The site was updated'


# -*- coding: utf8 -*-
from fabric.api import local, run, cd
from fabric import colors
from fabric.context_managers import settings
from fabric.contrib.console import confirm
from fabric.contrib import django
from fabric.utils import abort


class App(object):
    project_paths = {}
    project_package = None
    test_settings = None

    def __init__(self, project_paths, project_package, test_settings=None):
        self.project_paths = project_paths
        self.project_package = project_package
        self.test_settings = None
        django.project(project_package)

    def local_management_command(self, command, *args, **kwargs):
        return local("venv/bin/python manage.py %s" % command, *args, **kwargs)

    def run_management_command(self, instance, command):
        code_dir = self.project_paths[instance]
        with cd(code_dir):
            return run("venv/bin/python manage.py %s" % command)

    def test(self, is_deploying=True):
        with settings(warn_only=True):
            print(colors.yellow("Running tests, please wait!"))
            if settings is None:
                command = "test --settings=%s" % \
                    self.test_settings
            else:
                command = "test"
            result = self.local_management_command(command, capture=True)

        if result.failed:
            print(colors.red("Tests failed"))
            if is_deploying:
                if not confirm('Do you really want to deploy?'):
                    abort('')
        else:
            print(colors.green("All tests ok"))

    def run_server_updates(self, instance):
        code_dir = self.project_paths[instance]
        with cd(code_dir):
            run("git fetch")
            run("git reset --hard origin/master")

            run("venv/bin/pip install -r requirements.txt")

            from django.conf import settings
            if 'south' in settings.INSTALLED_APPS:
                self.run_management_command(instance,
                                            "syncdb --noinput --migrate")
            else:
                self.run_management_command(instance, "syncdb --noinput")

            if 'djangobower' in settings.INSTALLED_APPS:
                self.run_management_command(instance, "bower_install")

            self.run_management_command(instance, "collectstatic --noinput")

    def restart_app(self, instance):
        raise NotImplementedError

    def deploy(self, instance):
        self.run_server_updates(instance)
        self.restart_app(instance)

    def deploy_dev(self):
        if confirm("Do you want to run tests before deploying?"):
            self.test(is_deploying=True)

        self.deploy('dev')

    def deploy_prod(self, run_test=True):
        if run_test:
            self.test(is_deploying=True)

        self.deploy('prod')



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _
from ckeditor import fields
from sh.core.models import BaseModel

class NewsItem (BaseModel):
    title = models.CharField(max_length=250, verbose_name=_('title'))
    slug = models.SlugField(verbose_name=_('slug'), help_text=_('This is the text that will follow starthjelpa.no/nyheter/ in the URL'))
    pub_date = models.DateTimeField(verbose_name=_('publish date'))
    sticky = models.BooleanField(help_text=_('Check this if you want the news to appear on top of the news page'))
    content = fields.RichTextField(verbose_name=_('content'))

    class Meta:
        ordering = ('sticky', '-pub_date')

    def __unicode__(self):
        return self.title




from django.conf import settings
from django.template.base import Library

tracking_code = '<script type="text/javascript">' +\
                'var _gaq = _gaq || [];' +\
                '_gaq.push(["_setAccount", "%s"]);' % settings.ANALYTICS_CODE +\
                '_gaq.push(["_trackPageview"]);' +\
                '(function() {'+\
                'var ga = document.createElement("script"); ga.type = "text/javascript"; ga.async = true;' +\
                'ga.src = ("https:" == document.location.protocol ? "https://ssl" : "http://www") + ".google-analytics.com/ga.js";' +\
                'var s = document.getElementsByTagName("script")[0]; s.parentNode.insertBefore(ga, s);' +\
                '})();' +\
                '</script>'

register = Library()

@register.simple_tag(takes_context=True)
def analytics(context):
    user = context['user']
    if settings.ANALYTICS:
        if user.is_anonymous():
            return tracking_code
        elif settings.ANALYTICS_TRACK_ADMINS:
            return tracking_code

    return ''



# -*- coding: utf-8 -*-
from datetime import datetime
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.cache import cache
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404, render, redirect
from django.utils.translation import ugettext_lazy as _
from sh.app.news.forms import NewsItemForm
from sh.app.news.models import NewsItem

def news_feed(request):

    page = request.GET.get('page')

    news = cache.get('news')

    if news is None:
        news = NewsItem.objects.filter(pub_date__lte=datetime.now())

    paginator = Paginator(news, 10) # Show 25 contacts per page

    try:
        news = paginator.page(page)
    except PageNotAnInteger:
        news = paginator.page(1)
    except EmptyPage:
        news = paginator.page(paginator.num_pages)

    return render(request, 'news/feed.html', {
        'news': news,
    })

def view_item(request, item_slug):
    key = 'item%s' % item_slug
    item = cache.get(key)
    if item is None:
        item = get_object_or_404(NewsItem, is_active=True, slug=item_slug)
        cache.set(key, item)

    return render(request, "news/view_item.html", {
        'item': item,
        })

@login_required
def edit_item(request, id=None):

    if id is None:
        item = NewsItem()
    else:
        item = get_object_or_404(NewsItem, pk=id)

    form = NewsItemForm(instance=item)

    if request.method == 'POST':
        form = NewsItemForm(request.POST, instance=item)
        if form.is_valid():
            item = form.save()
            messages.success(request, _('News item successfully saved'))
            return redirect(reverse('view_item', args=[item.slug]))

    return render(request, 'news/edit_item.html', {
        'item': item,
        'form': form
    })



# -*- coding: utf-8 -*-
import re
from django.conf import settings
from django.template.base import Library
from sorl.thumbnail.shortcuts import get_thumbnail

register = Library()


@register.filter
def load_thumbnails(value):
    images = re.findall('<img( alt="([a-zA-Z0-9\.\:\/_ -]+)?")? src="([a-zA-Z0-9\.\:\/_ -]+)"([a-zA-Z0-9\.\:\;\/_=\" -]+)?>', value)
    if settings.DEBUG:
        url = "http://127.0.0.1:8000"
    else:
        url = settings.SITE_URL
    for i in images:

        width = re.search(r'width:( )?(\d+)px;', i[3])
        if width:
            width = int(width.group().replace('width: ', '').replace('px;', ''))
            if width > 1000:
                width = 1000

            value = value.replace(i[3], ' style="width: %d;"' % width)
        else:
            width = 1000

        float = re.search(r'float:([ ]+)?(\w+);', i[3])
        if float:
            value = value.replace('style=', 'class="pull-%s" style=' % float.group().replace('float:','').replace(';', '').strip())

        image = i[2]
        im = get_thumbnail("%s%s" % (url,image), '%d' % width)
        value = value.replace(image, im.url)
    return value


# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.shortcuts import redirect, get_object_or_404, render
from django.views.decorators.csrf import csrf_exempt
import time
from sh.core.forms import RegistrationForm
from sh.core.models import User

def register(request):
    form = RegistrationForm()
    if request.method == 'POST':
        form = RegistrationForm(request.POST)
        if form.is_valid():
            u = User(
                username=form.cleaned_data['email'],
                email=form.cleaned_data['email'],
                first_name=form.cleaned_data['first_name'],
                last_name=form.cleaned_data['last_name'],
            )
            u.save()
            u.generate_password()
            
            messages.success(request, 'The user was created successfully, check your email for the password')
            return redirect(reverse('django.contrib.auth.views.login'))

    return render(request, 'registration/register.html',
                  {'form': form})

@csrf_exempt
def backup_auto (request):
    if request.method == 'POST':
        u = get_object_or_404(User, username=request.POST.get('username'))
        if u.check_password(request.POST.get('password')):
            #TODO: Fill this
            name = ''
            db_username = ''
            with open('/django_projects/db_pw/', 'rb') as f:
                db_password = f.readline()
            db_password = db_password.replace("\n","").strip()
            import os
            path = os.path.join(settings.ROOTPATH, "backups/" + name + '-' + time.strftime('%y-%m-%d-%H-%M') + '.sql')
            command = 'mysqldump -h 127.0.0.1 -u ' + db_username + ' --password=' + db_password + ' ' + name + ' > ' + path
            result = os.system(command)
            if result > 0: return HttpResponse('Could not make a backup' + str(result) + '\n' + path)
            else:
                abspath = open(path,'r')
                response = HttpResponse(content=abspath.read())
                response['Content-Type']= 'application/octet-stream'
                response['Content-Disposition'] = 'attachment; filename=%s'\
                % os.path.basename(path)
            return response
    raise 404



# -*- coding: utf-8 -*-
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.http import HttpRequest
from django.template import defaultfilters
from django.utils.cache import get_cache_key


def slugify(value):
    value = value.replace(u"æ", 'ae')
    value = value.replace(u"ø", 'o')
    value = value.replace(u"å", 'aa')
    return defaultfilters.slugify(value)

def unslugify(value):
    value = value.replace('-', ' ')
    value = ''.join([value[0].upper(), value[1:]])
    return value

def expire_page_cache(view, args=None):
    """
    Removes cache created by cache_page functionality.
    Parameters are used as they are in reverse()
    """

    if args is None:
        path = reverse(view)
    else:
        path = reverse(view, args=args)

    request = HttpRequest()
    request.path = path
    key = get_cache_key(request)
    if cache.has_key(key):
        cache.delete(key)


# -*- coding: utf-8 -*-
from django import forms

class RegistrationForm (forms.Form):
    email = forms.EmailField(required=True)
    first_name = forms.CharField(required=True,max_length=30)
    last_name = forms.CharField(required=True, max_length=30)


# -*- coding: utf-8 -*-
from django.conf import settings

USER_ATTR_NAME = getattr(settings, 'LOCAL_USER_ATTR_NAME', '_current_user')

try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local
_thread_locals = local()

from new import instancemethod
def _do_set_current_user(user_fun):
    setattr(_thread_locals, USER_ATTR_NAME, instancemethod(user_fun, _thread_locals, type(_thread_locals)))

def _set_current_user(user=None):

    _do_set_current_user(lambda self: user)

class LocalUserMiddleware(object):
    def process_request(self, request):
        # request.user closure; asserts laziness; memoization is implemented in
        # request.user (non-data descriptor)
        _do_set_current_user(lambda self: getattr(request, 'user', None))
    #        if get_current_user().is_authenticated():
#            request.session['django_language'] = get_current_user().profile.language
#        else:
#            request.session['django_language'] = 'nb'

def get_current_user():
    current_user = getattr(_thread_locals, USER_ATTR_NAME, None)
    return current_user() if current_user else current_user


# -*- coding: utf-8 -*-
from datetime import datetime
from django.conf import settings
from sh.app.pages.models import Page, PageRevision
from sh.core.tests import TestCase
from django.test.client import Client
from django.core.urlresolvers import reverse

class PagesTest(TestCase):

    def setUp(self):
        super(PagesTest, self).setUp()
        self.frontpage = Page.objects.create(
            id=settings.FRONTPAGE_ID,
            created_by_id=1,
            saved_by_id=1,
            date_created=datetime(2012,02,02, 02, 02, 00),
            date_saved=datetime(2012,02,02, 02, 02, 00),
            title='Super frontpage',
            content='Masse fint innhold',
            weight=0
        )

        self.client = Client()

    def testViews(self):
        self.assertEqual(self.frontpage.slug, 'super-frontpage')
        self.assertEqual(PageRevision.objects.all().count(), 1)

        response_with_slug = self.client.get(reverse('view_page', args=[self.frontpage.slug]))
        self.assertEqual(response_with_slug.status_code, 200)

        response_fp = self.client.get(reverse('view_page'))
        self.assertEqual(response_fp.status_code, 200)

        self.assertEqual(response_with_slug.context['page'], response_fp.context['page'])

    def tearDown(self):
        self.frontpage.delete()
        super(PagesTest, self).tearDown()


from setuptools import find_packages, setup

setup(
    name='django-fabric',
    version='1.0',
    author='Rolf Erik Lekang',
    author_email='rolf@mocco.no',
    packages=find_packages(),

    include_package_data=True,
    zip_safe=False,
    install_requires=['fabric']
)



from sh.settings.base import ROOTPATH, MIDDLEWARE_CLASSES, INSTALLED_APPS

DEBUG=True
TEMPLATE_DEBUG=DEBUG

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ROOTPATH + '/project.db'
    }
}
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'unique-snowflake'
    }
}

#Django debug toolbar

EMAIL_HOST_USER = 'EMAIL'
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'


# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url

urlpatterns = patterns('sh.app.pages.views',
    url(r'^$', 'view_page', name='view_page'),
    url(r'^add/$', 'edit_page', name='add_page'),
    url(r'^(?P<id>\d+)/edit/$', 'edit_page', name='edit_page'),
    url(r'^(?P<id>\d+)/delete/$', 'delete_page', name='delete_page'),
    url(r'^(?P<page_slug>[a-z0-9-_]+)/$', 'view_page', name='view_page'),
    url(r'^(?P<category_slug>[a-z0-9-_]+)/(?P<page_slug>[a-z0-9-_]+)/$', 'view_page', name='view_page'),
)


# -*- coding: utf8 -*-
from sh.settings.base import *

try:
    from sh.settings.local import *
except ImportError, e:
    raise ImportError("Couldn't load local settings")

if DEBUG_TOOLBAR:
    MIDDLEWARE_CLASSES += ('debug_toolbar.middleware.DebugToolbarMiddleware',)
    from mocco.settings.debug_toolbar import *
    INSTALLED_APPS += ('debug_toolbar',)


# -*- coding: utf-8 -*-
from datetime import datetime
from django.conf import settings
from django.core.cache import cache
from django.utils.translation import ugettext_lazy as _
from django.db import models
from sh.core.models import BaseModel
from sh.core.util import slugify
from ckeditor import fields

class Category(BaseModel):
    title = models.CharField(max_length=25, verbose_name=_('title'))
    slug = models.CharField(max_length=30, editable=False)

    def __unicode__(self):
        return self.title

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title)
        super(Category, self).save(*args, **kwargs)


class PageRevision(BaseModel):
    revision_id = models.PositiveIntegerField(editable=False)
    page = models.ForeignKey('Page', editable=False, related_name='revisions')
    content = models.TextField(editable=False)
    date = models.DateTimeField(auto_now_add=True, editable=False)

    def __unicode__(self):
        return '%s revision %s' % (self.page.title, self.id)

    def save(self, *args, **kwargs):
        last = PageRevision.objects.filter(page=self.page)
        if last.count() > 0:
            self.revision_id = last[0].revision_id + 1
        else:
            self.revision_id = 1
        super(PageRevision, self).save(*args, **kwargs)

    @classmethod
    def create_revision(cls, page):
        r = PageRevision(page=page, content=page.content, date=datetime.now())
        r.save()


class Page(BaseModel):
    title = models.CharField(max_length=250, verbose_name=_('title'))
    menu_title = models.CharField(max_length=250, verbose_name=_('menu title'), help_text=_('This is the title that will be displayed in the menu'))
    slug = models.SlugField(verbose_name=_('slug'), help_text=_('This is the text that will follow starthjelpa.no in the URL'))
    weight = models.IntegerField(max_length=2, verbose_name=_('weight'), help_text=_('Lower weight will move the menu item to the front of the menu'))
    category = models.ForeignKey(Category, related_name='pages', null=True, blank=True, verbose_name=_('category'))
    content = fields.RichTextField(verbose_name=_('content'))

    class Meta:
        ordering = ('title',)

    def __unicode__(self):
        return self.title

    def save(self, *args, **kwargs):
        self.slug = slugify(self.menu_title)
        super(Page, self).save(*args, **kwargs)

        PageRevision.create_revision(self)

        key = "page%s" % self.slug
        cache.set(key, self)
        cache.delete('mainnav')

        if self.pk == settings.FRONTPAGE_ID:
            cache.set('frontpage', self)

    def delete(self, *args, **kwargs):
        """
        Must delete all page-revisions, since the foreignkey is not null.
        """
        for page_rev in self.revisions.all():
            page_rev.delete()
        cache.delete('page%s' % self.slug)
        cache.delete('mainnav')
        super(Page, self).delete(*args, **kwargs)




